@@ -53,7 +53,7 @@ BEGIN {
 };
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '1.004004';
+our $VERSION   = '1.008001';
 our @EXPORT    = qw( eval_closure );
 our @EXPORT_OK = qw(
 	HAS_LEXICAL_SUBS HAS_LEXICAL_VARS ALIAS_IMPLEMENTATION
@@ -61,6 +61,8 @@ our @EXPORT_OK = qw(
 	IMPLEMENTATION_NATIVE IMPLEMENTATION_TIE
 );
 
+$VERSION =~ tr/_//d;
+
 # See Types::TypeTiny for an explanation of this import method.
 #
 sub import {
@@ -192,6 +194,7 @@ sub _make_lexical_assignment {
 
 { my $tie; sub _manufacture_ties { $tie ||= eval <<'FALLBACK'; } }
 no warnings qw(void once uninitialized numeric);
+use Type::Tiny ();
 
 {
 	package #
@@ -215,12 +218,11 @@ no warnings qw(void once uninitialized numeric);
 			|| (defined tied(@$self) and tied(@$self)->can(@_));
 		return $code;
 	}
-	use overload
+	__PACKAGE__->Type::Tiny::_install_overloads(
 		q[bool]  => sub { !!   tied @{$_[0]} },
 		q[""]    => sub { '' . tied @{$_[0]} },
 		q[0+]    => sub { 0  + tied @{$_[0]} },
-		fallback => 1,
-	;
+	);
 }
 {
 	package #
@@ -244,12 +246,11 @@ no warnings qw(void once uninitialized numeric);
 			|| (defined tied(%$self) and tied(%$self)->can(@_));
 		return $code;
 	}
-	use overload
+	__PACKAGE__->Type::Tiny::_install_overloads(
 		q[bool]  => sub { !!   tied %{$_[0]} },
 		q[""]    => sub { '' . tied %{$_[0]} },
 		q[0+]    => sub { 0  + tied %{$_[0]} },
-		fallback => 1,
-	;
+	);
 }
 {
 	package #
@@ -273,12 +274,11 @@ no warnings qw(void once uninitialized numeric);
 			|| (defined tied($$self) and tied($$self)->can(@_));
 		return $code;
 	}
-	use overload
+	__PACKAGE__->Type::Tiny::_install_overloads(
 		q[bool]  => sub { !!   tied ${$_[0]} },
 		q[""]    => sub { '' . tied ${$_[0]} },
 		q[0+]    => sub { 0  + tied ${$_[0]} },
-		fallback => 1,
-	;
+	);
 }
 
 1;
@@ -299,9 +299,8 @@ __END__
 Eval::TypeTiny - utility to evaluate a string of Perl code in a clean environment
 
 =head1 STATUS
-
-This module is considered part of Type-Tiny's internals. It is not
-covered by the
+ 
+This module is covered by the
 L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
 
 =head1 DESCRIPTION
