@@ -6,9 +6,11 @@ use warnings;
 
 BEGIN {
 	$Type::Library::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Library::VERSION   = '1.002001';
+	$Type::Library::VERSION   = '1.008001';
 }
 
+$Type::Library::VERSION =~ tr/_//d;
+
 use Eval::TypeTiny qw< eval_closure >;
 use Scalar::Util qw< blessed refaddr >;
 use Type::Tiny;
@@ -108,7 +110,7 @@ sub _mksub
 		$type->qualified_name,
 		eval_closure(
 			source      => $source,
-			description => sprintf("exportable function '%s'", $type),
+			description => sprintf("exportable function '%s::%s'", $class, $type),
 			environment => {'$type' => \$type},
 		),
 	);
@@ -147,12 +149,49 @@ sub _exporter_expand_sub
 			$type, "is_$type", "assert_$type", "to_$type";
 	}
 	
-	if (my $type = $class->get_type($name))
+	my $typename = $name;
+	my $thingy   = undef;
+	if ($name =~ /^(is|assert|to)_(.+)$/)
 	{
-		my $post_method = q();
-		$post_method = '->mouse_type' if $globals->{mouse};
-		$post_method = '->moose_type' if $globals->{moose};
-		return ($name => $class->_mksub($type, $post_method)) if $post_method;
+		$thingy   = $1;
+		$typename = $2;
+	}
+	
+	if (my $type = $class->get_type($typename))
+	{
+		my $custom_type = 0;
+		for my $param (qw/ of where /)
+		{
+			exists $value->{$param} or next;
+			defined $value->{-as}
+				or _croak("Parameter '-as' not supplied");
+			$type = $type->$param($value->{$param});
+			$name = $value->{-as};
+			++$custom_type;
+		}
+		
+		if (!defined $thingy)
+		{
+			my $post_method = q();
+			$post_method = '->mouse_type' if $globals->{mouse};
+			$post_method = '->moose_type' if $globals->{moose};
+			return ($name => $class->_mksub($type, $post_method)) if $post_method || $custom_type;
+		}
+		elsif ($thingy eq 'is')
+		{
+			return ($value->{-as} || "is_$typename" => $type->compiled_check) if $custom_type;
+		}
+		elsif ($thingy eq 'assert')
+		{
+			return ($value->{-as} || "assert_$typename" => $type->_overload_coderef) if $custom_type;
+		}
+		elsif ($thingy eq 'to')
+		{
+			my $to_type = $type->has_coercion && $type->coercion->frozen
+				? $type->coercion->compiled_coercion
+				: sub ($) { $type->coerce($_[0]) };
+			return ($value->{-as} || "to_$typename" => $to_type) if $custom_type;
+		}
 	}
 	
 	return $class->SUPER::_exporter_expand_sub(@_);
@@ -164,8 +203,15 @@ sub _exporter_install_sub
 	my ($name, $value, $globals, $sym) = @_;
 	
 	my $package = $globals->{into};
+	my $type = $class->get_type($name);
 	
-	if (!ref $package and my $type = $class->get_type($name))
+	Exporter::Tiny::_carp(
+		"Exporting deprecated type %s to %s",
+		$type->qualified_name,
+		ref($package) ? "reference" : "package $package",
+	) if (defined $type and $type->deprecated and not $globals->{allow_deprecated});
+	
+	if (!ref $package and defined $type)
 	{
 		my ($prefix) = grep defined, $value->{-prefix}, $globals->{prefix}, q();
 		my ($suffix) = grep defined, $value->{-suffix}, $globals->{suffix}, q();
@@ -536,6 +582,11 @@ assume that the C<Types::Mine> library defines types C<Number> and C<String>.
    #
    use Types::Mine qw( :types );
    
+   # Exports "coerce_String" and "coerce_Number", as well as any other
+   # coercions
+   #
+   use Types::Mine qw( :coercions );
+   
    # Exports a sub "is_String" so that "is_String($foo)" is equivalent
    # to "String->check($foo)".
    #
@@ -594,7 +645,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
