@@ -6,35 +6,39 @@ use warnings;
 
 BEGIN {
 	if ($] < 5.008) { require Devel::TypeTiny::Perl56Compat };
+	if ($] < 5.010) { require Devel::TypeTiny::Perl58Compat };
 }
 
 BEGIN {
 	$Type::Tiny::AUTHORITY   = 'cpan:TOBYINK';
-	$Type::Tiny::VERSION     = '1.004004';
-	$Type::Tiny::XS_VERSION  = '0.011';
+	$Type::Tiny::VERSION     = '1.008001';
+	$Type::Tiny::XS_VERSION  = '0.016';
 }
 
-use Scalar::Util qw( blessed weaken refaddr isweak );
+$Type::Tiny::VERSION    =~ tr/_//d;
+$Type::Tiny::XS_VERSION =~ tr/_//d;
+
+use Scalar::Util qw( blessed );
 use Types::TypeTiny ();
 
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
 sub _swap { $_[2] ? @_[1,0] : @_[0,1] }
 
-BEGIN {
-	($] < 5.010001)
-		? eval q{ sub SUPPORT_SMARTMATCH () { !!0 } }
-		: eval q{ sub SUPPORT_SMARTMATCH () { !!1 } };
-	($] >= 5.014)
-		? eval q{ sub _FIXED_PRECEDENCE () { !!1 } }
-		: eval q{ sub _FIXED_PRECEDENCE () { !!0 } };
-};
+BEGIN {                                              # uncoverable statement
+	($] < 5.010001)                                   # uncoverable statement
+		? eval q{ sub SUPPORT_SMARTMATCH () { !!0 } }  # uncoverable statement
+		: eval q{ sub SUPPORT_SMARTMATCH () { !!1 } }; # uncoverable statement
+	($] >= 5.014)                                     # uncoverable statement
+		? eval q{ sub _FIXED_PRECEDENCE () { !!1 } }   # uncoverable statement
+		: eval q{ sub _FIXED_PRECEDENCE () { !!0 } };  # uncoverable statement
+};                                                   # uncoverable statement
 
 BEGIN {
-	my $try_xs =
-		exists($ENV{PERL_TYPE_TINY_XS}) ? !!$ENV{PERL_TYPE_TINY_XS} :
-		exists($ENV{PERL_ONLY})         ?  !$ENV{PERL_ONLY} :
-		1;
+	my $try_xs =                                                         # uncoverable statement
+		exists($ENV{PERL_TYPE_TINY_XS}) ? !!$ENV{PERL_TYPE_TINY_XS} :     # uncoverable statement
+		exists($ENV{PERL_ONLY})         ?  !$ENV{PERL_ONLY} :             # uncoverable statement
+		1;                                                                # uncoverable statement
 	
 	my $use_xs = 0;
 	$try_xs and eval {
@@ -52,8 +56,31 @@ BEGIN {
 		: sub () { !!0 };
 };
 
-use overload
-	q("")      => sub { caller =~ m{^(Moo::HandleMoose|Sub::Quote)} ? overload::StrVal($_[0]) : $_[0]->display_name },
+{
+	my $nil = sub {};
+	sub _install_overloads
+	{
+		no strict 'refs';
+		no warnings 'redefine', 'once';
+		# Coverage is checked on Perl 5.26
+		if ($] < 5.010) {              # uncoverable statement
+			require overload;           # uncoverable statement
+			push @_, fallback => 1;     # uncoverable statement
+			goto \&overload::OVERLOAD;  # uncoverable statement
+		};                             # uncoverable statement
+		my $class = shift;
+		*{$class . '::(('} = sub {};
+		*{$class . '::()'} = sub {};
+		*{$class . '::()'} = do { my $x = 1; \$x };
+		while (@_) {
+			my $f = shift;
+			*{$class . '::(' . $f} = ref $_[0] ? shift : do { my $m = shift; sub { shift->$m(@_) } };
+		}
+	}
+}
+
+__PACKAGE__->_install_overloads(
+	q("")      => sub { caller =~ m{^(Moo::HandleMoose|Sub::Quote)} ? $_[0]->_stringify_no_magic : $_[0]->display_name },
 	q(bool)    => sub { 1 },
 	q(&{})     => "_overload_coderef",
 	q(|)       => sub {
@@ -76,7 +103,7 @@ use overload
 				require Type::Tiny::_HalfOp;
 				return "Type::Tiny::_HalfOp"->new('|', @tc);
 			}
-	}
+		}
 		require Type::Tiny::Union;
 		return "Type::Tiny::Union"->new(type_constraints => \@tc)
 	},
@@ -113,35 +140,64 @@ use overload
 	q(>=)      => sub { my $m = $_[0]->can('is_a_type_of');  $m->(reverse _swap @_) },
 	q(eq)      => sub { "$_[0]" eq "$_[1]" },
 	q(cmp)     => sub { $_[2] ? ("$_[1]" cmp "$_[0]") : ("$_[0]" cmp "$_[1]") },
-	fallback   => 1,
-;
-BEGIN {
-	overload->import(
-		q(~~)    => sub { $_[0]->check($_[1]) },
-		fallback => 1, # 5.10 loses the fallback otherwise
-	) if Type::Tiny::SUPPORT_SMARTMATCH;
-}
+);
+	
+__PACKAGE__->_install_overloads(
+	q(~~)    => sub { $_[0]->check($_[1]) },
+) if Type::Tiny::SUPPORT_SMARTMATCH;
 
+# Would be easy to just return sub { $self->assert_return(@_) }
+# but try to build a more efficient coderef whenever possible.
+#
 sub _overload_coderef
 {
 	my $self = shift;
-	$self->message unless exists $self->{message};
 	
-#	if ($self->has_parent && $self->_is_null_constraint)
-#	{
-#		$self->{_overload_coderef} ||= $self->parent->_overload_coderef;
-#	}
-#	els
-	if (!exists($self->{message}) && exists(&Sub::Quote::quote_sub) && $self->can_be_inlined)
-	{
-		$self->{_overload_coderef} = Sub::Quote::quote_sub($self->inline_assert('$_[0]'))
-			if !$self->{_overload_coderef} || !$self->{_sub_quoted}++;
-	}
-	else
-	{
-		$self->{_overload_coderef} ||= sub { $self->assert_return(@_) };
-	}
+	# Bypass generating a coderef if we've already got the best possible one.
+	#
+	return $self->{_overload_coderef} if $self->{_overload_coderef_no_rebuild};
 	
+	# Subclasses of Type::Tiny might override assert_return to do some kind
+	# of interesting thing. In that case, we can't rely on it having identical
+	# behaviour to Type::Tiny::inline_assert.
+	#
+	$self->{_overrides_assert_return} = ($self->can('assert_return') != \&assert_return)
+		unless exists $self->{_overrides_assert_return};
+	
+	if ($self->{_overrides_assert_return}) {
+		$self->{_overload_coderef} ||= do {
+			Scalar::Util::weaken(my $weak = $self);
+			sub { $weak->assert_return(@_) };
+		};
+		++ $self->{_overload_coderef_no_rebuild};
+	}
+	elsif (exists(&Sub::Quote::quote_sub)) {
+		# Use `=` instead of `||=` because we want to overwrite non-Sub::Quote
+		# coderef if possible.
+		$self->{_overload_coderef} = $self->can_be_inlined
+			? Sub::Quote::quote_sub(
+				$self->inline_assert('$_[0]'),
+			)
+			: Sub::Quote::quote_sub(
+				$self->inline_assert('$_[0]', '$type'),
+				{ '$type' => \$self },
+			);
+		++ $self->{_overload_coderef_no_rebuild};
+	}
+	else {
+		require Eval::TypeTiny;
+		$self->{_overload_coderef} ||= $self->can_be_inlined
+			? Eval::TypeTiny::eval_closure(
+				source      => sprintf('sub { %s }', $self->inline_assert('$_[0]', undef, no_wrapper => 1)),
+				description => sprintf("compiled assertion 'assert_%s'", $self),
+			)
+			: Eval::TypeTiny::eval_closure(
+				source      => sprintf('sub { %s }', $self->inline_assert('$_[0]', '$type', no_wrapper => 1)),
+				description => sprintf("compiled assertion 'assert_%s'", $self),
+				environment => { '$type' => \$self },
+			);
+	}
+		
 	$self->{_overload_coderef};
 }
 
@@ -173,8 +229,7 @@ sub new
 		};
 	}
 	
-	if (exists $params{parent})
-	{
+	if (exists $params{parent}) {
 		$params{parent} = ref($params{parent}) =~ /^Type::Tiny\b/
 			? $params{parent}
 			: Types::TypeTiny::to_TypeTiny($params{parent});
@@ -182,8 +237,7 @@ sub new
 		_croak "Parent must be an instance of %s", __PACKAGE__
 			unless blessed($params{parent}) && $params{parent}->isa(__PACKAGE__);
 		
-		if ($params{parent}->deprecated and not exists $params{deprecated})
-		{
+		if ($params{parent}->deprecated and not exists $params{deprecated}) {
 			$params{deprecated} = 1;
 		}
 	}
@@ -194,16 +248,14 @@ sub new
 	$params{name} = "__ANON__" unless exists $params{name};
 	$params{uniq} = $uniq++;
 	
-	if ($params{name} ne "__ANON__")
-	{
+	if ($params{name} ne "__ANON__") {
 		# First try a fast ASCII-only expression, but fall back to Unicode
 		$params{name} =~ /^_{0,2}[A-Z][A-Za-z0-9_]+$/sm
 			or eval q( use 5.008; $params{name} =~ /^_{0,2}\p{Lu}[\p{L}0-9_]+$/sm )
 			or _croak '"%s" is not a valid type name', $params{name};
 	}
 	
-	if (exists $params{coercion} and !ref $params{coercion} and $params{coercion})
-	{
+	if (exists $params{coercion} and !ref $params{coercion} and $params{coercion}) {
 		$params{parent}->has_coercion
 			or _croak "coercion => 1 requires type to have a direct parent with a coercion";
 		
@@ -213,8 +265,7 @@ sub new
 	if (!exists $params{inlined}
 	and exists $params{constraint}
 	and ( !exists $params{parent} or $params{parent}->can_be_inlined )
-	and $QFS ||= "Sub::Quote"->can("quoted_from_sub"))
-	{
+	and $QFS ||= "Sub::Quote"->can("quoted_from_sub")) {
 		my (undef, $perlstring, $captures) = @{ $QFS->($params{constraint}) || [] };
 		
 		$params{inlined} = sub {
@@ -232,43 +283,44 @@ sub new
 	
 	my $self = bless \%params, $class;
 	
-	unless ($params{tmp})
-	{
+	unless ($params{tmp}) {
 		my $uniq = $self->{uniq};
 		
 		$ALL_TYPES{$uniq} = $self;
-		weaken( $ALL_TYPES{$uniq} );
+		Scalar::Util::weaken( $ALL_TYPES{$uniq} );
 		
-		package # no index
-			Moo::HandleMoose;
 		my $tmp = $self;
 		Scalar::Util::weaken($tmp);
-		$Moo::HandleMoose::TYPE_MAP{$self} = sub { $tmp };
+		$Moo::HandleMoose::TYPE_MAP{$self->_stringify_no_magic} = sub { $tmp };
 	}
 	
-	if (ref($params{coercion}) eq q(CODE))
-	{
+	if (ref($params{coercion}) eq q(CODE)) {
 		require Types::Standard;
 		my $code = delete($params{coercion});
 		$self->{coercion} = $self->_build_coercion;
 		$self->coercion->add_type_coercions(Types::Standard::Any(), $code);
 	}
-	elsif (ref($params{coercion}) eq q(ARRAY))
-	{
+	elsif (ref($params{coercion}) eq q(ARRAY)) {
 		my $arr = delete($params{coercion});
 		$self->{coercion} = $self->_build_coercion;
 		$self->coercion->add_type_coercions(@$arr);
 	}
-	
-	if ($params{my_methods})
-	{
+
+	# Documenting this here because it's too weird to be in the pod.
+	# There's a secret attribute called "_build_coercion" which takes a
+	# coderef. If present, then when $type->coercion is lazy built,
+	# the blank Type::Coercion object gets passed to the coderef,
+	# allowing the coderef to manipulate it a little. This is used by
+	# Types::TypeTiny to allow it to build a coercion for the TypeTiny
+	# type constraint without needing to load Type::Coercion yet.
+
+	if ($params{my_methods}) {
 		$subname =
 			eval { require Sub::Util } ? \&Sub::Util::set_subname :
 			eval { require Sub::Name } ? \&Sub::Name::subname :
 			0
 			if not defined $subname;
-		if ($subname)
-		{
+		if ($subname) {
 			(Scalar::Util::reftype($params{my_methods}{$_}) eq 'CODE') && $subname->(
 				sprintf("%s::my_%s", $self->qualified_name, $_),
 				$params{my_methods}{$_},
@@ -279,24 +331,24 @@ sub new
 	return $self;
 }
 
-sub DESTROY
-{
+sub DESTROY {
 	my $self = shift;
 	delete( $ALL_TYPES{$self->{uniq}} );
-	package # no index
-		Moo::HandleMoose;
-	delete( $Moo::HandleMoose::TYPE_MAP{$self} );
+	delete( $Moo::HandleMoose::TYPE_MAP{$self->_stringify_no_magic} );
 	return;
 }
 
-sub _clone
-{
+sub _clone {
 	my $self = shift;
 	my %opts;
 	$opts{$_} = $self->{$_} for qw< name display_name message >;
 	$self->create_child_type(%opts);
 }
 
+sub _stringify_no_magic {
+	sprintf('%s=%s(0x%08x)', blessed($_[0]), Scalar::Util::reftype($_[0]), Scalar::Util::refaddr($_[0]));
+}
+
 our $DD;
 sub _dd
 {
@@ -324,8 +376,7 @@ sub _dd
 	}
 }
 
-sub _loose_to_TypeTiny
-{
+sub _loose_to_TypeTiny {
 	map +(
 		ref($_)
 			? Types::TypeTiny::to_TypeTiny($_)
@@ -355,7 +406,6 @@ sub my_methods               { $_[0]{my_methods}     ||= $_[0]->_build_my_method
 
 sub has_parent               { exists $_[0]{parent} }
 sub has_library              { exists $_[0]{library} }
-sub has_coercion             {        $_[0]{coercion} and !!@{ $_[0]{coercion}->type_coercion_map } }
 sub has_inlined              { exists $_[0]{inlined} }
 sub has_constraint_generator { exists $_[0]{constraint_generator} }
 sub has_inline_generator     { exists $_[0]{inline_generator} }
@@ -366,12 +416,19 @@ sub has_deep_explanation     { exists $_[0]{deep_explanation} }
 
 sub _default_message         { $_[0]{_default_message} ||= $_[0]->_build_default_message }
 
+sub has_coercion
+{
+	$_[0]->coercion if $_[0]{_build_coercion}; # trigger auto build thing
+	$_[0]{coercion} and !!@{ $_[0]{coercion}->type_coercion_map }
+}
+
 sub _assert_coercion
 {
 	my $self = shift;
+	return $self->coercion if $self->{_build_coercion}; # trigger auto build thing
 	_croak "No coercion for this type constraint"
 		unless $self->has_coercion && @{$self->coercion->type_coercion_map};
-	return $self->coercion;
+	$self->coercion;
 }
 
 my $null_constraint = sub { !!1 };
@@ -397,12 +454,15 @@ sub _build_coercion
 	my $self = shift;
 	my %opts = (type_constraint => $self);
 	$opts{display_name} = "to_$self" unless $self->is_anon;
-	return "Type::Coercion"->new(%opts);
+	my $coercion = "Type::Coercion"->new(%opts);
+	$self->{_build_coercion}->($coercion) if ref $self->{_build_coercion};
+	$coercion;
 }
 
 sub _build_default_message
 {
 	my $self = shift;
+	$self->{is_using_default_message} = 1;
 	return sub { sprintf '%s did not pass type constraint', _dd($_[0]) } if "$self" eq "__ANON__";
 	my $name = "$self";
 	return sub { sprintf '%s did not pass type constraint "%s"', _dd($_[0]), $name };
@@ -421,6 +481,8 @@ sub _build_compiled_check
 {
 	my $self = shift;
 	
+	local our $AvoidCallbacks = 0;
+	
 	if ($self->_is_null_constraint and $self->has_parent)
 	{
 		return $self->parent->compiled_check;
@@ -448,62 +510,118 @@ sub _build_compiled_check
 	};
 }
 
+sub find_constraining_type
+{
+	my $self = shift;
+	if ($self->_is_null_constraint and $self->has_parent) {
+		return $self->parent->find_constraining_type;
+	}
+	$self;
+}
+
+our @CMP;
+
+sub CMP_SUPERTYPE          () { -1 }
+sub CMP_EQUAL              () {  0 }
+sub CMP_EQUIVALENT         () { '0E0' }
+sub CMP_SUBTYPE            () {  1 }
+sub CMP_UNKNOWN            () { ''; }
+
+# avoid getting mixed up with cmp operator at compile time
+*cmp = sub {
+	my ($A, $B) = _loose_to_TypeTiny($_[0], $_[1]);
+	return unless blessed($A) && $A->isa("Type::Tiny");
+	return unless blessed($B) && $B->isa("Type::Tiny");
+	for my $comparator (@CMP) {
+		my $result = $comparator->($A, $B);
+		next if $result eq CMP_UNKNOWN;
+		if ($result eq CMP_EQUIVALENT) {
+			my $prefer = @_==3 ? $_[2] : CMP_EQUAL;
+			return $prefer;
+		}
+		return $result;
+	}
+	return CMP_UNKNOWN;
+};
+
+push @CMP, sub {
+	my ($A, $B) = @_;
+	return CMP_EQUAL
+		if Scalar::Util::refaddr($A) == Scalar::Util::refaddr($B);
+	
+	return CMP_EQUIVALENT
+		if Scalar::Util::refaddr($A->compiled_check) == Scalar::Util::refaddr($B->compiled_check);
+
+	my $A_stem = $A->find_constraining_type;
+	my $B_stem = $B->find_constraining_type;
+	return CMP_EQUIVALENT
+		if Scalar::Util::refaddr($A_stem) == Scalar::Util::refaddr($B_stem);
+	return CMP_EQUIVALENT
+		if Scalar::Util::refaddr($A_stem->compiled_check) == Scalar::Util::refaddr($B_stem->compiled_check);
+	
+	if ($A_stem->can_be_inlined and $B_stem->can_be_inlined) {
+		return 0 if $A_stem->inline_check('$WOLFIE') eq $B_stem->inline_check('$WOLFIE');
+	}
+
+	A_IS_SUBTYPE: {
+		my $A_prime = $A_stem;
+		while ($A_prime->has_parent) {
+			$A_prime = $A_prime->parent;
+			return CMP_SUBTYPE
+				if Scalar::Util::refaddr($A_prime) == Scalar::Util::refaddr($B_stem);
+			return CMP_SUBTYPE
+				if Scalar::Util::refaddr($A_prime->compiled_check) == Scalar::Util::refaddr($B_stem->compiled_check);
+			if ($A_prime->can_be_inlined and $B_stem->can_be_inlined) {
+				return CMP_SUBTYPE
+					if $A_prime->inline_check('$WOLFIE') eq $B_stem->inline_check('$WOLFIE');
+			}
+		}
+	}
+
+	B_IS_SUBTYPE: {
+		my $B_prime = $B_stem;
+		while ($B_prime->has_parent) {
+			$B_prime = $B_prime->parent;
+			return CMP_SUPERTYPE
+				if Scalar::Util::refaddr($B_prime) == Scalar::Util::refaddr($A_stem);
+			return CMP_SUPERTYPE
+				if Scalar::Util::refaddr($B_prime->compiled_check) == Scalar::Util::refaddr($A_stem->compiled_check);
+			if ($A_stem->can_be_inlined and $B_prime->can_be_inlined) {
+				return CMP_SUPERTYPE
+					if $B_prime->inline_check('$WOLFIE') eq $A_stem->inline_check('$WOLFIE');
+			}
+		}
+	}
+	
+	return CMP_UNKNOWN;
+};
+
 sub equals
 {
-	my ($self, $other) = _loose_to_TypeTiny(@_);
-	return unless blessed($self)  && $self->isa("Type::Tiny");
-	return unless blessed($other) && $other->isa("Type::Tiny");
-	
-	return !!1 if refaddr($self) == refaddr($other);
-	
-	return !!1 if $self->has_parent  && $self->_is_null_constraint  && $self->parent==$other;
-	return !!1 if $other->has_parent && $other->_is_null_constraint && $other->parent==$self;
-	
-	return !!1 if refaddr($self->compiled_check) == refaddr($other->compiled_check);
-	
-	return $self->qualified_name eq $other->qualified_name
-		if $self->has_library && !$self->is_anon && $other->has_library && !$other->is_anon;
-	
-	return $self->inline_check('$x') eq $other->inline_check('$x')
-		if $self->can_be_inlined && $other->can_be_inlined;
-	
-	return;
+	my $result = Type::Tiny::cmp($_[0], $_[1]);
+	return unless defined $result;
+	$result eq CMP_EQUAL;
 }
 
 sub is_subtype_of
 {
-	my ($self, $other) = _loose_to_TypeTiny(@_);
-	return unless blessed($self)  && $self->isa("Type::Tiny");
-	return unless blessed($other) && $other->isa("Type::Tiny");
-
-#	my $this = $self;
-#	while (my $parent = $this->parent)
-#	{
-#		return !!1 if $parent->equals($other);
-#		$this = $parent;
-#	}
-#	return;
-
-	return unless $self->has_parent;
-	$self->parent->equals($other) or $self->parent->is_subtype_of($other);
+	my $result = Type::Tiny::cmp($_[0], $_[1], CMP_SUBTYPE);
+	return unless defined $result;
+	$result eq CMP_SUBTYPE;
 }
 
 sub is_supertype_of
 {
-	my ($self, $other) = _loose_to_TypeTiny(@_);
-	return unless blessed($self)  && $self->isa("Type::Tiny");
-	return unless blessed($other) && $other->isa("Type::Tiny");
-	
-	$other->is_subtype_of($self);
+	my $result = Type::Tiny::cmp($_[0], $_[1], CMP_SUBTYPE);
+	return unless defined $result;
+	$result eq CMP_SUPERTYPE;
 }
 
 sub is_a_type_of
 {
-	my ($self, $other) = _loose_to_TypeTiny(@_);
-	return unless blessed($self)  && $self->isa("Type::Tiny");
-	return unless blessed($other) && $other->isa("Type::Tiny");
-	
-	$self->equals($other) or $self->is_subtype_of($other);
+	my $result = Type::Tiny::cmp($_[0], $_[1]);
+	return unless defined $result;
+	$result eq CMP_SUBTYPE or $result eq CMP_EQUAL or $result eq CMP_EQUIVALENT;
 }
 
 sub strictly_equals
@@ -520,14 +638,6 @@ sub is_strictly_subtype_of
 	return unless blessed($self)  && $self->isa("Type::Tiny");
 	return unless blessed($other) && $other->isa("Type::Tiny");
 
-#	my $this = $self;
-#	while (my $parent = $this->parent)
-#	{
-#		return !!1 if $parent->strictly_equals($other);
-#		$this = $parent;
-#	}
-#	return;
-
 	return unless $self->has_parent;
 	$self->parent->strictly_equals($other) or $self->parent->is_strictly_subtype_of($other);
 }
@@ -673,6 +783,7 @@ sub _perlcode
 {
 	my $self = shift;
 	
+	local our $AvoidCallbacks = 1;
 	return $self->inline_check('$_')
 		if $self->can_be_inlined;
 	
@@ -745,29 +856,55 @@ sub inline_assert
 {
 	require B;
 	my $self = shift;
-	my $varname = $_[0];
-	my $code = sprintf(
-		q[do { no warnings "void"; %s ? %s : Type::Tiny::_failed_check(%d, %s, %s) };],
-		$self->inline_check(@_),
-		$varname,
-		$self->{uniq},
-		B::perlstring("$self"),
-		$varname,
-	);
-	return $code;
+	my ($varname, $typevarname, %extras) = @_;
+	
+	my $inline_check;
+	if ($self->can_be_inlined) {
+		$inline_check = sprintf('(%s)', $self->inline_check($varname));
+	}
+	elsif ($typevarname) {
+		$inline_check = sprintf('%s->check(%s)', $typevarname, $varname);
+	}
+	else {
+		_croak 'Cannot inline type constraint check for "%s"', $self;
+	}
+	
+	my $do_wrapper = !delete $extras{no_wrapper};
+	
+	my $inline_throw;
+	if ($typevarname) {
+		$inline_throw = sprintf(
+			'Type::Tiny::_failed_check(%s, %s, %s, %s)',
+			$typevarname,
+			B::perlstring("$self"),
+			$varname,
+			join(',', map +(B::perlstring($_) => B::perlstring($extras{$_})), sort keys %extras),
+		);
+	}
+	else {
+		$inline_throw = sprintf(
+			'Type::Tiny::_failed_check(%s, %s, %s, %s)',
+			$self->{uniq},
+			B::perlstring("$self"),
+			$varname,
+			join(',', map +(B::perlstring($_) => B::perlstring($extras{$_})), sort keys %extras),
+		);
+	}
+	
+	$do_wrapper
+		? qq[do { no warnings "void"; $inline_check or $inline_throw; $varname };]
+		: qq[     no warnings "void"; $inline_check or $inline_throw; $varname   ]
 }
 
-sub _failed_check
-{
+sub _failed_check {
 	require Error::TypeTiny::Assertion;
 	
 	my ($self, $name, $value, %attrs) = @_;
-	$self = $ALL_TYPES{$self} unless ref $self;
+	$self = $ALL_TYPES{$self} if defined $self && !ref $self;
 	
 	my $exception_class = delete($attrs{exception_class}) || "Error::TypeTiny::Assertion";
 	
-	if ($self)
-	{
+	if ($self) {
 		$exception_class->throw(
 			message => $self->get_message($value),
 			type    => $self,
@@ -775,8 +912,7 @@ sub _failed_check
 			%attrs,
 		);
 	}
-	else
-	{
+	else {
 		$exception_class->throw(
 			message => sprintf('%s did not pass type constraint "%s"', _dd($value), $name),
 			value   => $value,
@@ -807,69 +943,79 @@ sub is_parameterized
 	shift->has_parameters;
 }
 
-my %param_cache;
-sub parameterize
 {
-	my $self = shift;
-	
-	$self->is_parameterizable
-		or @_ ? _croak("Type '%s' does not accept parameters", "$self") : return($self);
-	
-	@_ = map Types::TypeTiny::to_TypeTiny($_), @_;
+	my %seen;
+	sub ____make_key {
+		join ',', map {
+			Types::TypeTiny::TypeTiny->check($_) ? sprintf('$Type::Tiny::ALL_TYPES{%d}', $_->{uniq}) :
+			ref() eq 'ARRAY'                     ? do { $seen{$_}++ ? '____CANNOT_KEY____' : sprintf('[%s]', ____make_key(@$_)) } :
+			ref() eq 'HASH'                      ? do { $seen{$_}++ ? '____CANNOT_KEY____' : sprintf('{%s}', ____make_key(%$_)) } :
+			ref() eq 'SCALAR' || ref() eq 'REF'  ? do { $seen{$_}++ ? '____CANNOT_KEY____' : sprintf('do { my $x = %s; \\$x }', ____make_key($$_)) } :
+			!defined()                           ? 'undef' :
+			!ref()                               ? do { require B; B::perlstring($_) } :
+			'____CANNOT_KEY____';
+		} @_;
+	}
+	my %param_cache;
+	sub parameterize
+	{
+		my $self = shift;
+		
+		$self->is_parameterizable
+			or @_ ? _croak("Type '%s' does not accept parameters", "$self") : return($self);
+		
+		@_ = map Types::TypeTiny::to_TypeTiny($_), @_;
 
-	# Generate a key for caching parameterized type constraints,
-	# but only if all the parameters are strings or type constraints.
-	my $key;
-	if ( not grep(ref($_) && !Types::TypeTiny::TypeTiny->check($_), @_) )
-	{
-		require B;
-		$key = join ":", map(Types::TypeTiny::TypeTiny->check($_) ? $_->{uniq} : B::perlstring($_), $self, @_);
-	}
-	
-	return $param_cache{$key} if defined $key && defined $param_cache{$key};
-	
-	local $Type::Tiny::parameterize_type = $self;
-	local $_ = $_[0];
-	my $P;
-	
-	my ($constraint, $compiled) = $self->constraint_generator->(@_);
-	
-	if (Types::TypeTiny::TypeTiny->check($constraint))
-	{
-		$P = $constraint;
-	}
-	else
-	{
-		my %options = (
-			constraint   => $constraint,
-			display_name => $self->name_generator->($self, @_),
-			parameters   => [@_],
-		);
-		$options{compiled_type_constraint} = $compiled
-			if $compiled;
-		$options{inlined} = $self->inline_generator->(@_)
-			if $self->has_inline_generator;
-		exists $options{$_} && !defined $options{$_} && delete $options{$_}
-			for keys %options;
+		# Generate a key for caching parameterized type constraints,
+		# but only if all the parameters are strings or type constraints.
+		%seen = ();
+		my $key = $self->____make_key(@_);
+		undef($key) if $key =~ /____CANNOT_KEY____/;
+		return $param_cache{$key} if defined $key && defined $param_cache{$key};
 		
-		$P = $self->create_child_type(%options);
+		local $Type::Tiny::parameterize_type = $self;
+		local $_ = $_[0];
+		my $P;
 		
-		my $coercion;
-		$coercion = $self->coercion_generator->($self, $P, @_)
-			if $self->has_coercion_generator;
-		$P->coercion->add_type_coercions( @{$coercion->type_coercion_map} )
-			if $coercion;
+		my ($constraint, $compiled) = $self->constraint_generator->(@_);
+		
+		if (Types::TypeTiny::TypeTiny->check($constraint))
+		{
+			$P = $constraint;
+		}
+		else
+		{
+			my %options = (
+				constraint   => $constraint,
+				display_name => $self->name_generator->($self, @_),
+				parameters   => [@_],
+			);
+			$options{compiled_type_constraint} = $compiled
+				if $compiled;
+			$options{inlined} = $self->inline_generator->(@_)
+				if $self->has_inline_generator;
+			exists $options{$_} && !defined $options{$_} && delete $options{$_}
+				for keys %options;
+			
+			$P = $self->create_child_type(%options);
+			
+			my $coercion;
+			$coercion = $self->coercion_generator->($self, $P, @_)
+				if $self->has_coercion_generator;
+			$P->coercion->add_type_coercions( @{$coercion->type_coercion_map} )
+				if $coercion;
+		}
+		
+		if (defined $key)
+		{
+			$param_cache{$key} = $P;
+			Scalar::Util::weaken($param_cache{$key});
+		}
+		
+		$P->coercion->freeze;
+		
+		return $P;
 	}
-	
-	if (defined $key)
-	{
-		$param_cache{$key} = $P;
-		weaken($param_cache{$key});
-	}
-	
-	$P->coercion->freeze;
-	
-	return $P;
 }
 
 sub child_type_class
@@ -880,14 +1026,16 @@ sub child_type_class
 sub create_child_type
 {
 	my $self = shift;
-	return $self->child_type_class->new(parent => $self, @_);
+	my %moreopts;
+	$moreopts{is_object} = 1 if $self->{is_object};
+	return $self->child_type_class->new(parent => $self, %moreopts, @_);
 }
 
 sub complementary_type
 {
 	my $self = shift;
 	my $r    = ($self->{complementary_type} ||= $self->_build_complementary_type);
-	weaken($self->{complementary_type}) unless isweak($self->{complementary_type});
+	Scalar::Util::weaken($self->{complementary_type}) unless Scalar::Util::isweak($self->{complementary_type});
 	return $r;
 }
 
@@ -901,6 +1049,8 @@ sub _build_complementary_type
 	$opts{display_name} =~ s/^\~{2}//;
 	$opts{inlined} = sub { shift; "not(".$self->inline_check(@_).")" }
 		if $self->can_be_inlined;
+	$opts{display_name} = $opts{name} = $self->{complement_name}
+		if $self->{complement_name};
 	return "Type::Tiny"->new(%opts);
 }
 
@@ -926,6 +1076,11 @@ sub _build_moose_type
 	}
 	else
 	{
+		# Type::Tiny is more flexible than Moose, allowing
+		# inlined to return a list. So we need to wrap the
+		# inlined coderef to make sure Moose gets a single
+		# string.
+		#
 		my $wrapped_inlined = sub {
 			shift;
 			$self->inline_check(@_);
@@ -1119,6 +1274,8 @@ sub _lookup_my_method
 	return;
 }
 
+my %object_methods = (with_attribute_values => 1, stringifies_to => 1, numifies_to => 1);
+
 sub can
 {
 	my $self = shift;
@@ -1141,6 +1298,11 @@ sub can
 			return $method if $method;
 		}
 	}
+
+	if ($self->{is_object} && $object_methods{$_[0]}) {
+		require Type::Tiny::ConstrainedObject;
+		return Type::Tiny::ConstrainedObject->can($_[0]);
+	}
 	
 	return;
 }
@@ -1161,10 +1323,17 @@ sub AUTOLOAD
 		if ($m =~ /\Amy_(.+)\z/)
 		{
 			my $method = $self->_lookup_my_method($1);
-			return $self->$method(@_) if $method;
+			return &$method($self, @_) if $method;
 		}
 	}
 	
+	if ($self->{is_object} && $object_methods{$m}) {
+		require Type::Tiny::ConstrainedObject;
+		unshift @_, $self;
+		no strict 'refs';
+		goto \&{"Type::Tiny::ConstrainedObject::$m"};
+	}
+	
 	_croak q[Can't locate object method "%s" via package "%s"], $m, ref($self)||$self;
 }
 
@@ -1197,13 +1366,15 @@ sub _actually_compile_type_constraint   { shift->_build_compiled_check }
 sub hand_optimized_type_constraint      { shift->{hand_optimized_type_constraint} }
 sub has_hand_optimized_type_constraint  { exists(shift->{hand_optimized_type_constraint}) }
 sub type_parameter             { (shift->parameters || [])->[0] }
+sub parameterized_from         { $_[0]->is_parameterized ? shift->parent : _croak("Not a parameterized type") }
+sub has_parameterized_from     { $_[0]->is_parameterized }
 
 # some stuff for Mouse-compatible API
 sub __is_parameterized         { shift->is_parameterized(@_) }
 sub _add_type_coercions        { shift->coercion->add_type_coercions(@_) };
 sub _as_string                 { shift->qualified_name(@_) }
 sub _compiled_type_coercion    { shift->coercion->compiled_coercion(@_) };
-sub _identity                  { refaddr(shift) };
+sub _identity                  { Scalar::Util::refaddr(shift) };
 sub _unite                     { require Type::Tiny::Union; "Type::Tiny::Union"->new(type_constraints => \@_) };
 
 # Hooks for Type::Tie
@@ -1227,6 +1398,74 @@ Type::Tiny - tiny, yet Moo(se)-compatible type constraint
 
 =head1 SYNOPSIS
 
+ use v5.12;
+ use strict;
+ use warnings;
+ 
+ package Horse {
+   use Moo;
+   use Types::Standard qw( Str Int Enum ArrayRef Object );
+   use Type::Params qw( compile );
+   use namespace::autoclean;
+   
+   has name => (
+     is       => 'ro',
+     isa      => Str,
+     required => 1,
+   );
+   has gender => (
+     is       => 'ro',
+     isa      => Enum[qw( f m )],
+   );
+   has age => (
+     is       => 'rw',
+     isa      => Int->where( '$_ >= 0' ),
+   );
+   has children => (
+     is       => 'ro',
+     isa      => ArrayRef[Object],
+     default  => sub { return [] },
+   );
+   
+   sub add_child {
+     state $check = compile( Object, Object );  # method signature
+     
+     my ($self, $child) = $check->(@_);         # unpack @_
+     push @{ $self->children }, $child;
+     
+     return $self;
+   }
+ }
+ 
+ package main;
+ 
+ my $boldruler = Horse->new(
+   name    => "Bold Ruler",
+   gender  => 'm',
+   age     => 16,
+ );
+ 
+ my $secretariat = Horse->new(
+   name    => "Secretariat",
+   gender  => 'm',
+   age     => 0,
+ );
+ 
+ $boldruler->add_child( $secretariat );
+
+=head1 STATUS
+
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
+=head1 DESCRIPTION
+
+This documents the internals of the L<Type::Tiny> class. L<Type::Tiny::Manual>
+is a better starting place if you're new.
+
+L<Type::Tiny> is a small class for creating Moose-like type constraint
+objects which are compatible with Moo, Moose and Mouse.
+
    use Scalar::Util qw(looks_like_number);
    use Type::Tiny;
    
@@ -1251,22 +1490,9 @@ Type::Tiny - tiny, yet Moo(se)-compatible type constraint
       has favourite_number => (is => "ro", isa => $NUM);
    }
 
-=head1 STATUS
-
-This module is covered by the
-L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
-
-=head1 DESCRIPTION
-
-L<Type::Tiny> is a tiny class for creating Moose-like type constraint
-objects which are compatible with Moo, Moose and Mouse.
-
 Maybe now we won't need to have separate MooseX, MouseX and MooX versions
 of everything? We can but hope...
 
-This documents the internals of L<Type::Tiny>. L<Type::Tiny::Manual> is
-a better starting place if you're new.
-
 =head2 Constructor
 
 =over
@@ -1316,6 +1542,11 @@ If provided, must be a Type::Tiny object.
 A coderef which returns a string of Perl code suitable for inlining this
 type. Optional.
 
+(The coderef will be called in list context and can actually return
+a list of strings which will be joined with C<< && >>. If the first item
+on the list is undef, it will be substituted with the type's parent's
+inline check.)
+
 If C<constraint> (above) is a coderef generated via L<Sub::Quote>, then
 Type::Tiny I<may> be able to automatically generate C<inlined> for you.
 If C<constraint> (above) is a string, it will be able to.
@@ -1507,6 +1738,10 @@ Returns true iff the type constraint does not have a C<name>.
 Indicates whether a type has been parameterized (e.g. C<< ArrayRef[Int] >>)
 or could potentially be (e.g. C<< ArrayRef >>).
 
+=item C<< has_parameterized_from >>
+
+Useless alias for C<is_parameterized>.
+
 =back
 
 =head3 Validation and coercion
@@ -1659,7 +1894,7 @@ Returns a list of all this type constraint's ancestor constraints. For
 example, if called on the C<Str> type constraint would return the list
 C<< (Value, Defined, Item, Any) >>.
 
-B<< Due to a historical misunderstanding, this differs from the Moose
+I<< Due to a historical misunderstanding, this differs from the Moose
 implementation of the C<parents> method. In Moose, C<parents> only returns the
 immediate parent type constraints, and because type constraints only have
 one immediate parent, this is effectively an alias for C<parent>. The
@@ -1678,6 +1913,15 @@ being checked is C<< $_ >>. Returns undef if there is no match.
 In list context also returns the number of type constraints which had
 been looped through before the matching constraint was found.
 
+=item C<< find_constraining_type >>
+
+Finds the nearest ancestor type constraint (including the type itself)
+which has a C<constraint> coderef.
+
+Equivalent to:
+
+   $type->find_parent(sub { not $_->_is_null_constraint })
+
 =item C<< coercibles >>
 
 Return a type constraint which is the union of type constraints that can be
@@ -1693,12 +1937,54 @@ parameters; otherwise returns undef. For example:
    ( ArrayRef[Int] )->parent;            # returns ArrayRef
 
 Note that parameterizable type constraints can perfectly legitimately take
-multiple parameters (several off the parameterizable type constraints in
+multiple parameters (several of the parameterizable type constraints in
 L<Types::Standard> do). This method only returns the first such parameter.
 L</"Attributes related to parameterizable and parameterized types">
 documents the C<parameters> attribute, which returns an arrayref of all
 the parameters.
 
+=item C<< parameterized_from >>
+
+Harder to spell alias for C<parent> that only works for parameterized
+types.
+
+=back
+
+I<< Hint for people subclassing Type::Tiny: >>
+Since version 1.006000, the methods for determining subtype, supertype, and
+type equality should I<not> be overridden in subclasses of Type::Tiny. This
+is because of the problem of diamond inheritance. If X and Y are both
+subclasses of Type::Tiny, they I<both> need to be consulted to figure out
+how type constraints are related; not just one of them should be overriding
+these methods. See the source code for L<Type::Tiny::Enum> for an example of
+how subclasses can give hints about type relationships to Type::Tiny.
+Summary: push a coderef onto C<< @Type::Tiny::CMP >>. This coderef will be
+passed two type constraints. It should then return one of the constants
+Type::Tiny::CMP_SUBTYPE (first type is a subtype of second type),
+Type::Tiny::CMP_SUPERTYPE (second type is a subtype of first type),
+Type::Tiny::CMP_EQUAL (the two types are exactly the same),
+Type::Tiny::CMP_EQUIVALENT (the two types are effectively the same), or
+Type::Tiny::CMP_UNKNOWN (your coderef couldn't establish any relationship).
+
+=head3 Type relationship introspection function
+
+=over
+
+=item C<< Type::Tiny::cmp($type1, $type2) >>
+
+The subtype/supertype relationship between types results in a partial
+ordering of type constraints.
+
+This function will return one of the constants:
+Type::Tiny::CMP_SUBTYPE (first type is a subtype of second type),
+Type::Tiny::CMP_SUPERTYPE (second type is a subtype of first type),
+Type::Tiny::CMP_EQUAL (the two types are exactly the same),
+Type::Tiny::CMP_EQUIVALENT (the two types are effectively the same), or
+Type::Tiny::CMP_UNKNOWN (couldn't establish any relationship).
+In numeric contexts, these evaluate to -1, 1, 0, 0, and 0, making it
+potentially usable with C<sort> (though you may need to silence warnings
+about treating the empty string as a numeric value).
+
 =back
 
 =head3 Inlining methods
@@ -1731,10 +2017,15 @@ For Moose-compat, there is an alias C<< _inline_check >> for this method.
 
 Much like C<inline_check> but outputs a statement of the form:
 
-   die ... unless ...;
+   ... or die ...;
 
-Note that if this type has a custom error message, the inlined code will
-I<ignore> this custom message!!
+Can also be called line C<< inline_assert($varname, $typevarname, %extras) >>.
+In this case, it will generate a string of code that may include
+C<< $typevarname >> which is supposed to be the name of a variable holding
+the type itself. (This is kinda complicated, but it allows a useful string
+to still be produced if the type is not inlineable.) The C<< %extras >> are
+additional options to be passed to L<Error::TypeTiny::Assertion>'s constructor
+and must be key-value pairs of strings only, no references or undefs.
 
 =back
 
@@ -1867,6 +2158,28 @@ dumps. (Default limit is 72.)
 This is a package variable (rather than get/set class methods) to allow
 for easy localization.
 
+=item C<< $Type::Tiny::AvoidCallbacks >>
+
+If this variable is set to true (you should usually do it in a
+C<local> scope), it acts as a hint for type constraints, when
+generating inlined code, to avoid making any callbacks to
+variables and functions defined outside the inlined code itself.
+
+This should have the effect that C<< $type->inline_check('$foo') >>
+will return a string of code capable of checking the type on
+Perl installations that don't have Type::Tiny installed. This
+is intended to allow Type::Tiny to be used with things like
+L<Mite>.
+
+The variable works on the honour system. Types need to explicitly
+check it and decide to generate different code based on its
+truth value. The bundled types in L<Types::Standard>,
+L<Types::Common::Numeric>, and L<Types::Common::String> all do.
+(B<StrMatch> is sometimes unable to, and will issue a warning
+if it needs to rely on callbacks when asked not to.)
+
+Most normal users can ignore this.
+
 =back
 
 =head2 Environment
@@ -1886,13 +2199,10 @@ of parts of Type::Tiny.
 Please report any bugs to
 L<http://rt.cpan.org/Dist/Display.html?Queue=Type-Tiny>.
 
-=head1 SUPPORT
-
-B<< IRC: >> support is available through in the I<< #moops >> channel
-on L<irc.perl.org|http://www.irc.perl.org/channels.html>.
-
 =head1 SEE ALSO
 
+L<The Type::Tiny homepage|http://typetiny.toby.ink/>.
+
 L<Type::Tiny::Manual>, L<Type::API>.
 
 L<Type::Library>, L<Type::Utils>, L<Types::Standard>, L<Type::Coercion>.
@@ -1905,6 +2215,12 @@ L<Mouse::Meta::TypeConstraint>.
 
 L<Type::Params>.
 
+L<Type::Tiny on GitHub|https://github.com/tobyink/p5-type-tiny>,
+L<Type::Tiny on Travis-CI|https://travis-ci.org/tobyink/p5-type-tiny>,
+L<Type::Tiny on AppVeyor|https://ci.appveyor.com/project/tobyink/p5-type-tiny>,
+L<Type::Tiny on Codecov|https://codecov.io/gh/tobyink/p5-type-tiny>,
+L<Type::Tiny on Coveralls|https://coveralls.io/github/tobyink/p5-type-tiny>.
+
 =head1 AUTHOR
 
 Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
