@@ -6,23 +6,23 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Duck::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Duck::VERSION   = '1.004004';
+	$Type::Tiny::Duck::VERSION   = '1.008001';
 }
 
+$Type::Tiny::Duck::VERSION =~ tr/_//d;
+
 use Scalar::Util qw< blessed >;
 
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-use Type::Tiny ();
-our @ISA = 'Type::Tiny';
+require Type::Tiny::ConstrainedObject;
+our @ISA = 'Type::Tiny::ConstrainedObject';
+sub _short_name { 'Duck' }
 
 sub new {
 	my $proto = shift;
 	
 	my %opts = (@_==1) ? %{$_[0]} : @_;
-	_croak "Duck type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
-	_croak "Duck type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
-	_croak "Duck type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
 	_croak "Need to supply list of methods" unless exists $opts{methods};
 	
 	$opts{methods} = [$opts{methods}] unless ref $opts{methods};
@@ -48,6 +48,8 @@ sub inlined     { $_[0]{inlined} ||= $_[0]->_build_inlined }
 
 sub has_inlined { !!1 }
 
+sub _is_null_constraint { 0 }
+
 sub _build_constraint
 {
 	my $self    = shift;
@@ -91,17 +93,6 @@ sub _instantiate_moose_type
 	return "Moose::Meta::TypeConstraint::DuckType"->new(%opts, methods => $self->methods);
 }
 
-sub has_parent
-{
-	!!1;
-}
-
-sub parent
-{
-	require Types::Standard;
-	Types::Standard::Object();
-}
-
 sub validate_explain
 {
 	my $self = shift;
@@ -124,6 +115,33 @@ sub validate_explain
 	];
 }
 
+push @Type::Tiny::CMP, sub {
+	my $A = shift->find_constraining_type;
+	my $B = shift->find_constraining_type;
+	return Type::Tiny::CMP_UNKNOWN unless $A->isa(__PACKAGE__) && $B->isa(__PACKAGE__);
+	
+	my %seen;
+	for my $word (@{$A->methods}) {
+		$seen{$word} += 1;
+	}
+	for my $word (@{$B->methods}) {
+		$seen{$word} += 2;
+	}
+	
+	my $values = join('', CORE::values %seen);
+	if ($values =~ /^3*$/) {
+		return Type::Tiny::CMP_EQUIVALENT;
+	}
+	elsif ($values !~ /2/) {
+		return Type::Tiny::CMP_SUBTYPE;
+	}
+	elsif ($values !~ /1/) {
+		return Type::Tiny::CMP_SUPERTYPE;
+	}
+	
+	return Type::Tiny::CMP_UNKNOWN;
+};
+
 1;
 
 __END__
@@ -168,11 +186,29 @@ Instead rely on the default.
 
 =item C<parent>
 
-Parent is always Types::Standard::Object, and cannot be passed to the
+Parent is always B<Types::Standard::Object>, and cannot be passed to the
 constructor.
 
 =back
 
+=head2 Methods
+
+=over
+
+=item C<< stringifies_to($constraint) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=item C<< numifies_to($constraint) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=item C<< with_attribute_values($attr1 => $constraint1, ...) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=back
+
 =head1 BUGS
 
 Please report any bugs to
