@@ -6,19 +6,23 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Intersection::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Intersection::VERSION   = '1.002001';
+	$Type::Tiny::Intersection::VERSION   = '1.008001';
 }
 
+$Type::Tiny::Intersection::VERSION =~ tr/_//d;
+
 use Scalar::Util qw< blessed >;
 use Types::TypeTiny ();
 
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-use overload q[@{}] => sub { $_[0]{type_constraints} ||= [] };
-
 use Type::Tiny ();
 our @ISA = 'Type::Tiny';
 
+__PACKAGE__->_install_overloads(
+	q[@{}] => sub { $_[0]{type_constraints} ||= [] },
+);
+
 sub new {
 	my $proto = shift;
 	
@@ -57,6 +61,8 @@ sub new {
 sub type_constraints { $_[0]{type_constraints} }
 sub constraint       { $_[0]{constraint} ||= $_[0]->_build_constraint }
 
+sub _is_null_constraint { 0 }
+
 sub _build_display_name
 {
 	my $self = shift;
@@ -145,6 +151,65 @@ sub validate_explain
 	return;  # uncoverable statement
 }
 
+my $_delegate = sub {
+	my ($self, $method) = (shift, shift);
+	my @types = @{ $self->type_constraints };
+	my $found = 0;
+	for my $i (0 .. $#types) {
+		my $type = $types[$i];
+		if ($type->can($method)) {
+			$types[$i] = $type->$method(@_);
+			++$found;
+			last;
+		}
+	}
+	_croak('Could not apply method %s to any type within the intersection', $method) unless $found;
+	ref($self)->new(type_constraints => \@types);
+};
+
+sub stringifies_to {
+	my $self = shift;
+	$self->$_delegate(stringifies_to => @_);
+}
+
+sub numifies_to {
+	my $self = shift;
+	$self->$_delegate(numifies_to => @_);
+}
+
+sub with_attribute_values {
+	my $self = shift;
+	$self->$_delegate(with_attribute_values => @_);
+}
+
+my $comparator;
+$comparator = sub {
+	my $A = shift->find_constraining_type;
+	my $B = shift->find_constraining_type;
+	
+	if ($A->isa(__PACKAGE__)) {
+		my @A_constraints = map $_->find_constraining_type, @{ $A->type_constraints };
+		
+		my @A_equal_to_B = grep $_->equals($B), @A_constraints;
+		if (@A_equal_to_B == @A_constraints) {
+			return Type::Tiny::CMP_EQUIVALENT();
+		}
+		
+		my @A_subs_of_B = grep $_->is_a_type_of($B), @A_constraints;
+		if (@A_subs_of_B) {
+			return Type::Tiny::CMP_SUBTYPE();
+		}
+	}
+	
+	elsif ($B->isa(__PACKAGE__)) {
+		my $r = $comparator->($B, $A);
+		return  $r if $r eq Type::Tiny::CMP_EQUIVALENT();
+		return -$r if $r eq Type::Tiny::CMP_SUBTYPE();
+	}
+	
+	return Type::Tiny::CMP_UNKNOWN();
+};
+push @Type::Tiny::CMP, $comparator;
 
 1;
 
@@ -202,6 +267,24 @@ parent type; we choose the first arbitrarily.)
 
 =back
 
+=head2 Methods
+
+=over
+
+=item C<< stringifies_to($constraint) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=item C<< numifies_to($constraint) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=item C<< with_attribute_values($attr1 => $constraint1, ...) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=back
+
 =head2 Overloading
 
 =over
@@ -231,7 +314,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
