@@ -6,27 +6,25 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Role::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Role::VERSION   = '1.004004';
+	$Type::Tiny::Role::VERSION   = '1.008001';
 }
 
+$Type::Tiny::Role::VERSION =~ tr/_//d;
+
 use Scalar::Util qw< blessed weaken >;
 
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-require Type::Tiny;
-our @ISA = 'Type::Tiny';
+require Type::Tiny::ConstrainedObject;
+our @ISA = 'Type::Tiny::ConstrainedObject';
+sub _short_name { 'Role' }
 
 my %cache;
 
 sub new {
 	my $proto = shift;
-	
 	my %opts = (@_==1) ? %{$_[0]} : @_;
-	_croak "Role type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
-	_croak "Role type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
-	_croak "Role type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
 	_croak "Need to supply role name" unless exists $opts{role};
-	
 	return $proto->SUPER::new(%opts);
 }
 
@@ -35,6 +33,8 @@ sub inlined     { $_[0]{inlined} ||= $_[0]->_build_inlined }
 
 sub has_inlined { !!1 }
 
+sub _is_null_constraint { 0 }
+
 sub _build_constraint
 {
 	my $self = shift;
@@ -61,17 +61,6 @@ sub _build_default_message
 	return sub { sprintf '%s did not pass type constraint "%s" (not DOES %s)', Type::Tiny::_dd($_[0]), $name, $c };
 }
 
-sub has_parent
-{
-	!!1;
-}
-
-sub parent
-{
-	require Types::Standard;
-	Types::Standard::Object();
-}
-
 sub validate_explain
 {
 	my $self = shift;
@@ -138,11 +127,29 @@ Instead rely on the default.
 
 =item C<parent>
 
-Parent is always Types::Standard::Object, and cannot be passed to the
+Parent is always B<Types::Standard::Object>, and cannot be passed to the
 constructor.
 
 =back
 
+=head2 Methods
+
+=over
+
+=item C<< stringifies_to($constraint) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=item C<< numifies_to($constraint) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=item C<< with_attribute_values($attr1 => $constraint1, ...) >>
+
+See L<Type::Tiny::ConstrainedObject>.
+
+=back
+
 =head1 BUGS
 
 Please report any bugs to
