@@ -6,10 +6,10 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::_HalfOp::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::_HalfOp::VERSION   = '1.004004';
+	$Type::Tiny::_HalfOp::VERSION   = '1.008001';
 }
 
-use overload ();
+$Type::Tiny::_HalfOp::VERSION =~ tr/_//d;
 
 sub new {
 	my ($class, $op, $param, $type) = @_;
@@ -21,6 +21,7 @@ sub new {
 }
 
 sub complete {
+	require overload;
 	my ($self, $type) = @_;
 	my $complete_type = $type->parameterize(@{$self->{param}});
 	my $method = overload::Method($complete_type, $self->{op});
