@@ -6,10 +6,12 @@ use warnings;
 
 BEGIN {
 	$Type::Utils::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Utils::VERSION   = '0.022';
+	$Type::Utils::VERSION   = '1.008001';
 }
 
-sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
+$Type::Utils::VERSION =~ tr/_//d;
+
+sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
 use Scalar::Util qw< blessed >;
 use Type::Library;
@@ -27,11 +29,13 @@ our @EXPORT_OK = (
 	qw<
 		extends type subtype
 		match_on_type compile_match_on_type
-		dwim_type
+		dwim_type english_list
+		classifier
 	>,
 );
 
-use base qw< Exporter::TypeTiny >;
+require Exporter::Tiny;
+our @ISA = 'Exporter::Tiny';
 
 sub extends
 {
@@ -40,8 +44,58 @@ sub extends
 	
 	foreach my $lib (@_)
 	{
-		eval "require $lib" or _croak "Could not load library '$lib': $@";
-		$caller->add_type($lib->get_type($_)) for $lib->meta->type_names;
+		eval "use $lib; 1" or _croak "Could not load library '$lib': $@";
+		
+		if ($lib->isa("Type::Library") or $lib eq 'Types::TypeTiny')
+		{
+			$caller->add_type( $lib->get_type($_) )
+				for sort $lib->meta->type_names;
+			$caller->add_coercion( $lib->get_coercion($_) )
+				for sort $lib->meta->coercion_names;
+		}
+		elsif ($lib->isa('MooseX::Types::Base'))
+		{
+			require Moose::Util::TypeConstraints;
+			my $types = $lib->type_storage;
+			for my $name (sort keys %$types)
+			{
+				my $moose = Moose::Util::TypeConstraints::find_type_constraint($types->{$name});
+				my $tt    = Types::TypeTiny::to_TypeTiny($moose);
+				my $c     = $moose->has_coercion && @{ $moose->coercion->type_coercion_map || [] };
+				$caller->add_type(
+					$tt->create_child_type(library => $caller, name => $name, coercion => $c ? 1 : 0)
+				);
+			}
+		}
+		elsif ($lib->isa('MouseX::Types::Base'))
+		{
+			require Mouse::Util::TypeConstraints;
+			my $types = $lib->type_storage;
+			for my $name (sort keys %$types)
+			{
+				my $mouse = Mouse::Util::TypeConstraints::find_type_constraint($types->{$name});
+				my $tt    = Types::TypeTiny::to_TypeTiny($mouse);
+				$caller->add_type(
+					$tt->create_child_type(library => $caller, name => $name, coercion => $mouse->has_coercion ? 1 : 0)
+				);
+			}
+		}
+		elsif ($lib->isa('Specio::Exporter'))
+		{
+			my $types = $lib->Specio::Registry::exportable_types_for_package;
+			for my $name (sort keys %$types)
+			{
+				my $specio = $types->{$name};
+				my $tt     = Types::TypeTiny::to_TypeTiny($specio);
+				$caller->add_type(
+					$tt->create_child_type(library => $caller, name => $name)
+				);
+			}
+		}
+		else
+		{
+			_croak("'$lib' is not a type constraint library");
+		}
 	}
 }
 
@@ -51,6 +105,11 @@ sub declare
 	if (@_ % 2 == 0)
 	{
 		%opts = @_;
+		if (@_==2 and $_[0]=~ /^_*[A-Z]/ and $_[1] =~ /^[0-9]+$/)
+		{
+			require Carp;
+			Carp::carp("Possible missing comma after 'declare $_[0]'");
+		}
 	}
 	else
 	{
@@ -121,12 +180,14 @@ sub inline_as (&;@)
 sub class_type
 {
 	my $name = ref($_[0]) ? undef : shift;
-	my %opts = %{ +shift };
+	my %opts = %{ shift or {} };
 	
 	if (defined $name)
 	{
 		$opts{name}  = $name unless exists $opts{name};
 		$opts{class} = $name unless exists $opts{class};
+		
+		$opts{name} =~ s/:://g;
 	}
 	
 	$opts{bless} = "Type::Tiny::Class";
@@ -138,12 +199,14 @@ sub class_type
 sub role_type
 {
 	my $name = ref($_[0]) ? undef : shift;
-	my %opts = %{ +shift };
+	my %opts = %{ shift or {} };
 	
 	if (defined $name)
 	{
-		$opts{name}  = $name unless exists $opts{name};
-		$opts{role}  = $name unless exists $opts{role};
+		$opts{name} = $name unless exists $opts{name};
+		$opts{role} = $name unless exists $opts{role};
+		
+		$opts{name} =~ s/:://g;
 	}
 	
 	$opts{bless} = "Type::Tiny::Role";
@@ -155,7 +218,7 @@ sub role_type
 sub duck_type
 {
 	my $name    = ref($_[0]) ? undef : shift;
-	my @methods = @{ +shift };
+	my @methods = @{ shift or [] };
 	
 	my %opts;
 	$opts{name} = $name if defined $name;
@@ -170,7 +233,7 @@ sub duck_type
 sub enum
 {
 	my $name   = ref($_[0]) ? undef : shift;
-	my @values = @{ +shift };
+	my @values = @{ shift or [] };
 	
 	my %opts;
 	$opts{name} = $name if defined $name;
@@ -185,7 +248,7 @@ sub enum
 sub union
 {
 	my $name = ref($_[0]) ? undef : shift;
-	my @tcs  = @{ +shift };
+	my @tcs  = @{ shift or [] };
 	
 	my %opts;
 	$opts{name} = $name if defined $name;
@@ -200,7 +263,7 @@ sub union
 sub intersection
 {
 	my $name = ref($_[0]) ? undef : shift;
-	my @tcs  = @{ +shift };
+	my @tcs  = @{ shift or [] };
 	
 	my %opts;
 	$opts{name} = $name if defined $name;
@@ -241,6 +304,10 @@ sub declare_coercion
 			push @C, shift;
 		}
 	}
+	else
+	{
+		@C = @_;
+	}
 	
 	$c->add_type_coercions(@C);
 	
@@ -296,20 +363,17 @@ sub match_on_type
 	
 	while (@_)
 	{
-		my ($type, $code);
+		my $code;
 		if (@_ == 1)
 		{
-			require Types::Standard;
-			($type, $code) = (Types::Standard::Any(), shift);
+			$code = shift;
 		}
 		else
 		{
-			($type, $code) = splice(@_, 0, 2);
-			TypeTiny->($type);
+			(my($type), $code) = splice(@_, 0, 2);
+			TypeTiny->($type)->check($value) or next;
 		}
 		
-		$type->check($value) or next;
-		
 		if (StringLike->check($code))
 		{
 			local $_ = $value;
@@ -384,7 +448,7 @@ sub compile_match_on_type
 		}
 	}
 	
-	push @code, 'else', '  { Type::Util::_croak("No cases matched for %s", Type::Tiny::_dd($_[0])) }';
+	push @code, 'else', '  { Type::Utils::_croak("No cases matched for %s", Type::Tiny::_dd($_[0])) }';
 	
 	push @code, '}';  # /sub
 	
@@ -398,12 +462,80 @@ sub compile_match_on_type
 	);
 }
 
+sub classifier
+{
+	my $i;
+	compile_match_on_type(
+		+(
+			map {
+				my $type = $_->[0];
+				$type => sub { $type };
+			}
+			sort { $b->[1] <=> $a->[1] or $a->[2] <=> $b->[2] }
+			map [$_, scalar(my @parents = $_->parents), ++$i],
+			@_
+		),
+		q[ undef ],
+	);
+}
+
 {
 	package #hide
 	Type::Registry::DWIM;
 	
 	our @ISA = qw(Type::Registry);
 	
+	sub foreign_lookup
+	{
+		my $self = shift;
+		my $r = $self->SUPER::foreign_lookup(@_);
+		return $r if $r;
+		
+		if (my $assume = $self->{"~~assume"}
+		and $_[0] =~ /[A-Z_a-z][0-9A-Z_a-z]*(?:::[0-9A-Z_a-z]+)*/)
+		{
+			my @methods = ref($assume) ? @$assume : $assume;
+			
+			for my $method (@methods)
+			{
+				$r = $self->$method(@_);
+				return $r if $r;
+			}
+		}
+		
+		return;
+	}
+	
+	sub lookup_via_moose
+	{
+		my $self = shift;
+		
+		if ($INC{'Moose.pm'})
+		{
+			require Moose::Util::TypeConstraints;
+			require Types::TypeTiny;
+			my $r = Moose::Util::TypeConstraints::find_type_constraint($_[0]);
+			return Types::TypeTiny::to_TypeTiny($r) if defined $r;
+		}
+		
+		return;
+	}
+	
+	sub lookup_via_mouse
+	{
+		my $self = shift;
+		
+		if ($INC{'Mouse.pm'})
+		{
+			require Mouse::Util::TypeConstraints;
+			require Types::TypeTiny;
+			my $r = Mouse::Util::TypeConstraints::find_type_constraint($_[0]);
+			return Types::TypeTiny::to_TypeTiny($r) if defined $r;
+		}
+		
+		return;
+	}
+	
 	sub simple_lookup
 	{
 		my $self = shift;
@@ -426,44 +558,26 @@ sub compile_match_on_type
 		# Only continue any further if we've been called from Type::Parser.
 		return unless $_[1];
 		
-		# If Moose is loaded...
-		if ($INC{'Moose.pm'})
+		my $meta;
+		if (defined $self->{"~~chained"})
 		{
-			require Moose::Util::TypeConstraints;
-			require Types::TypeTiny;
-			$r = Moose::Util::TypeConstraints::find_type_constraint($_[0]);
-			return Types::TypeTiny::to_TypeTiny($r) if defined $r;
+			$meta ||= Moose::Util::find_meta($self->{"~~chained"}) if $INC{'Moose.pm'};
+			$meta ||= Mouse::Util::find_meta($self->{"~~chained"}) if $INC{'Mouse.pm'};
 		}
 		
-		# If Mouse is loaded...
-		if ($INC{'Mouse.pm'})
+		if ($meta and $meta->isa('Class::MOP::Module'))
 		{
-			require Mouse::Util::TypeConstraints;
-			require Types::TypeTiny;
-			$r = Mouse::Util::TypeConstraints::find_type_constraint($_[0]);
-			return Types::TypeTiny::to_TypeTiny($r) if defined $r;
+			$r = $self->lookup_via_moose(@_);
+			return $r if $r;
 		}
 		
-		return unless $_[0] =~ /^\s*(\w+(::\w+)*)\s*$/sm;
-		return unless defined $self->{"~~assume"};
-		
-		# Lastly, if it looks like a class/role name, assume it's
-		# supposed to be a class/role type.
-		#
-		
-		if ($self->{"~~assume"} eq "Type::Tiny::Class")
+		elsif ($meta and $meta->isa('Mouse::Meta::Module'))
 		{
-			require Type::Tiny::Class;
-			return "Type::Tiny::Class"->new(class => $_[0]);
+			$r = $self->lookup_via_mouse(@_);
+			return $r if $r;
 		}
 		
-		if ($self->{"~~assume"} eq "Type::Tiny::Role")
-		{
-			require Type::Tiny::Role;
-			return "Type::Tiny::Role"->new(role => $_[0]);
-		}
-		
-		die;
+		return $self->foreign_lookup(@_);
 	}
 }
 
@@ -479,9 +593,32 @@ sub dwim_type
 	};
 	
 	local $dwimmer->{'~~chained'} = $opts{for};
-	local $dwimmer->{'~~assume'}  = $opts{does} ? 'Type::Tiny::Role' : 'Type::Tiny::Class';
+	local $dwimmer->{'~~assume'}  = $opts{fallback} || [
+		qw/ lookup_via_moose lookup_via_mouse /,
+		$opts{does} ? 'make_role_type' : 'make_class_type',
+	];
 	
-	$dwimmer->lookup($string);
+	local $@ = undef;
+	my $type;
+	unless (eval { $type = $dwimmer->lookup($string); 1 })
+	{
+		my $e = $@;
+		die($e) unless $e =~ /not a known type constraint/;
+	}
+	
+	$type;
+}
+
+sub english_list
+{
+	my $conjunction = ref($_[0]) eq 'SCALAR' ? ${+shift} : 'and';
+	my @items = sort @_;
+	
+	return $items[0] if @items == 1;
+	return "$items[0] $conjunction $items[1]" if @items == 2;
+	
+	my $tail = pop @items;
+	join(', ', @items, "$conjunction $tail");
 }
 
 1;
@@ -515,6 +652,11 @@ Type::Utils - utility functions to make defining and using type constraints a li
    coerce "AllCaps",
       from "Str", via { uc($_) };
 
+=head1 STATUS
+
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
 =head1 DESCRIPTION
 
 This module provides utility functions to make defining and using type
@@ -538,13 +680,13 @@ specify the parent type (if any) and (possibly) refine its definition.
 
    my $EvenInt = declare as Int, where { $_ % 2 == 0 };
 
-B<< NOTE: >>
+I<< NOTE: >>
 If the caller package inherits from L<Type::Library> then any non-anonymous
 types declared in the package will be automatically installed into the
 library.
 
 Hidden gem: if you're inheriting from a type constraint that includes some
-coercions, you can include C<< coerce => 1 >> in the C<< %options >> hash
+coercions, you can include C<< coercion => 1 >> in the C<< %options >> hash
 to inherit the coercions.
 
 =item C<< subtype $name, %options >>
@@ -620,7 +762,7 @@ significant performance improvements.
    
    warn EvenInt->inline_check('$xxx');  # demonstration
 
-B<Experimental:> your C<inline_as> block can return a list, in which case
+Your C<inline_as> block can return a list, in which case
 these will be smushed together with "&&". The first item on the list may
 be undef, in which case the undef will be replaced by the inlined parent
 type constraint. (And will throw an exception if there is no parent.)
@@ -632,22 +774,34 @@ type constraint. (And will throw an exception if there is no parent.)
          return (undef, "($_ % 2 == 0)");
       };
 
-Returning a list like this is considered experimental, is not tested very
-much, and I offer no guarantees that it will necessarily work with
-Moose/Mouse/Moo.
-
 =item C<< class_type $name, { class => $package, %options } >>
 
 =item C<< class_type { class => $package, %options } >>
 
+=item C<< class_type $name >>
+
 Shortcut for declaring a L<Type::Tiny::Class> type constraint.
 
+If C<< $package >> is omitted, is assumed to be the same as C<< $name >>.
+If C<< $name >> contains "::" (which would be an invalid name as far as
+L<Type::Tiny> is concerned), this will be removed.
+
+So for example, C<< class_type("Foo::Bar") >> declares a L<Type::Tiny::Class>
+type constraint named "FooBar" which constrains values to objects blessed
+into the "Foo::Bar" package.
+
 =item C<< role_type $name, { role => $package, %options } >>
 
 =item C<< role_type { role => $package, %options } >>
 
+=item C<< role_type $name >>
+
 Shortcut for declaring a L<Type::Tiny::Role> type constraint.
 
+If C<< $package >> is omitted, is assumed to be the same as C<< $name >>.
+If C<< $name >> contains "::" (which would be an invalid name as far as
+L<Type::Tiny> is concerned), this will be removed.
+
 =item C<< duck_type $name, \@methods >>
 
 =item C<< duck_type \@methods >>
@@ -824,6 +978,30 @@ are good for this. (Same sort of idea as L<Type::Params>.)
 
 This function is not exported by default.
 
+=item C<< my $coderef = classifier(@types) >>
+
+Returns a coderef that can be used to classify values according to their
+type constraint. The coderef, when passed a value, returns a type
+constraint which the value satisfies.
+
+   use feature qw( say );
+   use Type::Utils qw( classifier );
+   use Types::Standard qw( Int Num Str Any );
+   
+   my $classifier = classifier(Str, Int, Num, Any);
+   
+   say $classifier->( "42"  )->name;   # Int
+   say $classifier->( "4.2" )->name;   # Num
+   say $classifier->( []    )->name;   # Any
+
+Note that, for example, "42" satisfies Int, but it would satisfy the
+type constraints Num, Str, and Any as well. In this case, the
+classifier has picked the most specific type constraint that "42"
+satisfies.
+
+If no type constraint is satisfied by the value, then the classifier
+will return undef.
+
 =item C<< dwim_type($string, %options) >>
 
 Given a string like "ArrayRef[Int|CodeRef]", turns it into a type constraint
@@ -831,25 +1009,54 @@ object, hopefully doing what you mean.
 
 It uses the syntax of L<Type::Parser>. Firstly the L<Type::Registry>
 for the caller package is consulted; if that doesn't have a match,
-L<Types::Standard> is consulted for type constraint names; and if
-there's still no match, then if a type constraint looks like a class
-name, a new L<Type::Tiny::Class> object is created for it.
+L<Types::Standard> is consulted for standard type constraint names.
 
-Somewhere along the way, it also checks Moose/Mouse's type constraint
-registries if they are loaded.
+If none of the above yields a type constraint, and the caller class
+is a Moose-based class, then C<dwim_type> attempts to look the type
+constraint up in the Moose type registry. If it's a Mouse-based class,
+then the Mouse type registry is used instead.
+
+If no type constraint can be found via these normal methods, several
+fallbacks are available:
+
+=over
+
+=item C<lookup_via_moose>
+
+Lookup in Moose registry even if caller is non-Moose class.
+
+=item C<lookup_via_mouse>
+
+Lookup in Mouse registry even if caller is non-Mouse class.
+
+=item C<make_class_type>
+
+Create a new Type::Tiny::Class constraint.
+
+=item C<make_role_type>
+
+Create a new Type::Tiny::Role constraint.
+
+=back
+
+You can alter which should be attempted, and in which order, by passing
+an option to C<dwim_type>:
+
+   my $type = Type::Utils::dwim_type(
+      "ArrayRef[Int]",
+      fallback      => [ "lookup_via_mouse" , "make_role_type" ],
+   );
+
+For historical reasons, by default the fallbacks attempted are:
+
+   lookup_via_moose, lookup_via_mouse, make_class_type
+
+You may set C<fallback> to an empty arrayref to avoid using any of
+these fallbacks.
 
 You can specify an alternative for the caller using the C<for> option.
-If you'd rather create a L<Type::Tiny::Role> object, set the C<does>
-option to true.
 
-   # An arrayref of objects, each of which must do role Foo.
-   my $type = dwim_type("ArrayRef[Foo]", does => 1);
-   
-   Type::Registry->for_me->add_types("-Standard");
-   Type::Registry->for_me->alias_type("Int" => "Foo");
-   
-   # An arrayref of integers.
-   my $type = dwim_type("ArrayRef[Foo]", does => 1);
+   my $type = dwim_type("ArrayRef", for => "Moose::Object");
 
 While it's probably better overall to use the proper L<Type::Registry>
 interface for resolving type constraint strings, this function often does
@@ -861,15 +1068,26 @@ return undef.
 
 This function is not exported by default.
 
+=item C<< english_list(\$conjunction, @items) >>
+
+Joins the items with commas, placing a conjunction before the final item.
+The conjunction is optional, defaulting to "and".
+
+   english_list(qw/foo bar baz/);       # "foo, bar, and baz"
+   english_list(\"or", qw/quux quuux/); # "quux or quuux"
+
+This function is not exported by default.
+
 =back
 
 =head1 EXPORT
 
 By default, all of the functions documented above are exported, except
 C<subtype> and C<type> (prefer C<declare> instead), C<extends>, C<dwim_type>,
-and C<match_on_type>/C<compile_match_on_type>.
+C<match_on_type>/C<compile_match_on_type>, C<classifier>, and
+C<english_list>.
 
-This module uses L<Exporter::TypeTiny>; see the documentation of that module
+This module uses L<Exporter::Tiny>; see the documentation of that module
 for tips and tricks importing from Type::Utils.
 
 =head1 BUGS
@@ -895,7 +1113,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
