@@ -6,9 +6,11 @@ use warnings;
 
 BEGIN {
 	$Types::Standard::ArrayRef::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Standard::ArrayRef::VERSION   = '1.004004';
+	$Types::Standard::ArrayRef::VERSION   = '1.008001';
 }
 
+$Types::Standard::ArrayRef::VERSION =~ tr/_//d;
+
 use Type::Tiny ();
 use Types::Standard ();
 use Types::TypeTiny ();
@@ -25,18 +27,19 @@ sub __constraint_generator
 	Types::TypeTiny::TypeTiny->check($param)
 		or _croak("Parameter to ArrayRef[`a] expected to be a type constraint; got $param");
 	
-	_croak("Only one parameter to ArrayRef[`a] expected; got @{[ 1 + @_ ]}. Did you mean to use Tuple[`a]?")
-		if @_;
+	my ($min, $max) = (0, -1);
+	$min = Types::Standard::Int->assert_return(shift) if @_;
+	$max = Types::Standard::Int->assert_return(shift) if @_;
 	
 	my $param_compiled_check = $param->compiled_check;
 	my $xsub;
-	if (Type::Tiny::_USE_XS)
+	if (Type::Tiny::_USE_XS and $min==0 and $max==-1)
 	{
 		my $paramname = Type::Tiny::XS::is_known($param_compiled_check);
 		$xsub = Type::Tiny::XS::get_coderef_for("ArrayRef[$paramname]")
 			if $paramname;
 	}
-	elsif (Type::Tiny::_USE_MOUSE and $param->_has_xsub)
+	elsif (Type::Tiny::_USE_MOUSE and $param->_has_xsub and $min==0 and $max==-1)
 	{
 		require Mouse::Util::TypeConstraints;
 		my $maker = "Mouse::Util::TypeConstraints"->can("_parameterize_ArrayRef_for");
@@ -44,33 +47,69 @@ sub __constraint_generator
 	}
 	
 	return (
-		sub
-		{
+		sub {
 			my $array = shift;
 			$param->check($_) || return for @$array;
 			return !!1;
 		},
 		$xsub,
-	);
+	) if $min==0 and $max==-1;
+	
+	return sub {
+		my $array = shift;
+		return if @$array < $min;
+		$param->check($_) || return for @$array;
+		return !!1;
+	} if $max==-1;
+	
+	return sub {
+		my $array = shift;
+		return if @$array > $max;
+		$param->check($_) || return for @$array;
+		return !!1;
+	} if $min==0;
+	
+	return sub {
+		my $array = shift;
+		return if @$array < $min;
+		return if @$array > $max;
+		$param->check($_) || return for @$array;
+		return !!1;
+	};
 }
 
 sub __inline_generator
 {
 	my $param = shift;
+	my ($min, $max) = (0, -1);
+	$min = shift if @_;
+	$max = shift if @_;
 	
 	my $param_compiled_check = $param->compiled_check;
-	if (Type::Tiny::_USE_XS)
+	my $xsubname;
+	if (Type::Tiny::_USE_XS and $min==0 and $max==-1)
 	{
 		my $paramname = Type::Tiny::XS::is_known($param_compiled_check);
-		my $xsubname  = Type::Tiny::XS::get_subname_for("ArrayRef[$paramname]");
-		return sub { "$xsubname\($_[1]\)" } if $xsubname;
+		$xsubname  = Type::Tiny::XS::get_subname_for("ArrayRef[$paramname]");
 	}
 	
 	return unless $param->can_be_inlined;
-	my $param_check = $param->inline_check('$i');
+	
 	return sub {
 		my $v = $_[1];
+		return "$xsubname\($v\)" if $xsubname && !$Type::Tiny::AvoidCallbacks;
 		my $p = Types::Standard::ArrayRef->inline_check($v);
+		
+		if ($min != 0) {
+			$p .= sprintf(' and @{%s} >= %d', $v, $min);
+		}
+		if ($max > 0) {
+			$p .= sprintf(' and @{%s} <= %d', $v, $max);
+		}
+		
+		my $param_check = $param->inline_check('$i');
+		return $p if $param->{uniq} eq Types::Standard::Any->{uniq};
+		
 		"$p and do { "
 		.  "my \$ok = 1; "
 		.  "for my \$i (\@{$v}) { "
@@ -85,6 +124,23 @@ sub __deep_explanation
 {
 	my ($type, $value, $varname) = @_;
 	my $param = $type->parameters->[0];
+	my ($min, $max) = (0, -1);
+	$min = $type->parameters->[1] if @{$type->parameters} > 1;  # TODO
+	$max = $type->parameters->[2] if @{$type->parameters} > 2;  # TODO
+	
+	if ($min != 0 and @$value < $min) {
+		return [
+			sprintf('"%s" constrains array length at least %s', $type, $min),
+			sprintf('@{%s} is %d', $varname, scalar @$value),
+		];
+	}
+	
+	if ($max > 0 and @$value > $max) {
+		return [
+			sprintf('"%s" constrains array length at most %d', $type, $max),
+			sprintf('@{%s} is %d', $varname, scalar @$value),
+		];
+	}
 	
 	for my $i (0 .. $#$value)
 	{
@@ -93,13 +149,14 @@ sub __deep_explanation
 		return [
 			sprintf('"%s" constrains each value in the array with "%s"', $type, $param),
 			@{ $param->validate_explain($item, sprintf('%s->[%d]', $varname, $i)) },
-		]
+		];
 	}
 	
 	# This should never happen...
 	return;  # uncoverable statement
 }
 
+# TODO: min and max???
 sub __coercion_generator
 {
 	my ($parent, $child, $param) = @_;
